/* 
 *========================================================================
 * 
 * Copyright 2005 Vincent Massol & Matt Smith.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *========================================================================
 */
package patterntesting.plugin.aspectj;

import java.io.*;
import java.util.*;

import org.aspectj.lang.reflect.SourceLocation;


/**
 * @author <a href="mailto:matt@ojojo.com">Matt Smith</a>
 *
 * @version $Id: Logger.java,v 1.3 2007/12/08 21:47:39 oboehm Exp $
 */
public class Logger
{
   /**
    * Lists of errors that were logged. Array of
    * {@link AjcFileResult} objects.
    */
   private Hashtable<String, AjcFileResult> errors = new Hashtable<String, AjcFileResult>();
   private Hashtable results = new Hashtable();
   private File resultFile;
   private static Logger LOGGER = new Logger();
   
   private Logger()
   {
      this.setResultFile();
      try
      {
         this.write(new FileOutputStream(this.resultFile));
      }
      catch(FileNotFoundException fnfe)
      {
         fnfe.printStackTrace();
      }
   }
   
   public static Logger getInstance()
   {
      return LOGGER;
   }
   
   /**
    * Set "/patterntesting/patterntesting.xml" as default result file
    * if there is a directory "/patterntesting". Otherwise use the
    * tmp dir to store it.
    */
   private void setResultFile() {
	   File dir = new File("/patterntesting");
	   if (!dir.isDirectory()) {
		   File tmpdir = new File(System.getProperty("java.io.tmpdir"));
		   dir = new File(tmpdir, dir.getName());
		   dir.mkdirs();
	   }
	   this.setResultFile(new File(dir, "patterntesting.xml"));
   }
   
   public void setResultFile(File file)
   {
      this.resultFile = file;
   }
   
   public File getResultFile() {
	   return this.resultFile;
   }
   
   /**
    * @return the list of errors that were logged as
    *          a List of AjcResult objects
    */
   public Enumeration getErrors()
   {
      return this.errors.elements();
   }
   
   public int countErrors() {
	   int n = 0;
	   for (Enumeration e = getErrors(); e.hasMoreElements(); e.nextElement()) {
		   n++;
	   }
	   return n;
   }
   
   public Enumeration getReportResults()
   {
      return this.results.elements();
   }
   
   /**
    * 
    * @param location the SourceLocation of the error
    * @param message the message to log
    */
   public synchronized void logPTViolation(SourceLocation location, String message)
   {
      if(location != null)
      {
          String path = location.getFileName();
          AjcFileResult fileResult = 
              (AjcFileResult)errors.get(path);
          if(fileResult == null)
          {
              fileResult = new AjcFileResult(path);
              errors.put(path, fileResult);
          }
          fileResult.addError(new AjcResult(location.getLine(), message));
          FileOutputStream out = null;
          try
          {
             out = new FileOutputStream(resultFile);
             this.write(out);
             out.flush();
             out.close();
             out = null;
          }
          catch(Exception e)
          {
             e.printStackTrace();
          }
          finally
          {
             if(out != null)
             {
                try
                {
                   out.flush();
                   out.close();
                }
                catch(IOException ie)
                {
                   //do nothing
                }
             }
          }
          
      }
   }
   
   public void write(OutputStream resultStream)
   {
      XMLFormatter formatter = new XMLFormatter();
      try
      {
         resultStream.write(formatter.format(
               getReportResults(),
               getErrors()).getBytes());
      }
      catch(IOException ioe)
      {
         ioe.printStackTrace();
      }
   }
}
